{ Copyright (C) 2012 H Page-Clark

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA. }

unit ParseWordsTestCase;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, parseWords, fpcunit, testregistry;

const longText = 'We find the need, in the course of the development of a project '+
  'for a function that takes text (of arbitrary length, such as a paragraph from '+
  'this book), and processes it by parsing it into its constituent words.';
type

  { TParseWordsTestCase }

  TParseWordsTestCase= class(TTestCase)
  published
    procedure BoundToSucceed;
    procedure ParseEmptyString;  // ''
    procedure ParseNoSeparatorsInString; //'ThereIsNoSeparatorInThisString'
    procedure ParseOnlySeparatorsInString; // ';,.()!'
    procedure ParseLeadingAndTrailingSeparators; //'____LeadingAndTrailing.....'
    procedure ParseSeparatorsAllTogether; // 'start.,.,.,.,.end'
    procedure ParseLongText; // longText - see const above
  end;

implementation

procedure TParseWordsTestCase.BoundToSucceed;
begin
 AssertNull('This is bound to succeed', nil);
end;

procedure TParseWordsTestCase.ParseEmptyString;
begin
 AssertNull('ParseEmptyString', ParseToWords('', []));
end;

procedure TParseWordsTestCase.ParseNoSeparatorsInString;
begin
 AssertEquals('ParseNoSeparatorsInString', 'ThereIsNoSeparatorInThisString',
               ParseToWords('ThereIsNoSeparatorInThisString', [])[0]);
end;

procedure TParseWordsTestCase.ParseOnlySeparatorsInString;
begin
 AssertEquals('ParseOnlySeparatorsInString', 0, ParseToWords(';,.()!',[]).Count);
end;

procedure TParseWordsTestCase.ParseLeadingAndTrailingSeparators;
begin
  AssertEquals('ParseLeadingAndTrailingSeparators', 'LeadingAndTrailing',
                ParseToWords('____LeadingAndTrailing.....', [])[0]);
end;

procedure TParseWordsTestCase.ParseSeparatorsAllTogether;
begin
  AssertEquals('ParseSeparatorsAllTogether', 'start',
                ParseToWords('start.,.,.,.,.end', [])[0]);
end;

procedure TParseWordsTestCase.ParseLongText;
begin
  AssertEquals('ParseLongText', 39, ParseToWords(longText, []).Count);
end;



initialization

  RegisterTest(TParseWordsTestCase);
end.

